; Fall Down by David Phillips <david@acz.org>

#include "gbc.inc"

;============================================================================
BLOCKSPR		= $60
CHECKSUM		= $fa11
CURSORSPR		= 12

StackTop		= _BANKV - 2
Vars			= _BANK0
SaveRam			= _EEW

KeyP 			= Vars			; [1]
KeyC			= Vars + 1		; [1]
TempStr			= Vars + 2		; [6]
BallY			= Vars + 8		; [1]
BallX			= Vars + 9		; [1]
BallAnim		= Vars + 10		; [1]
MapRow			= Vars + 11		; [1]
RandSeed		= Vars + 12		; [2]
Score			= Vars + 14		; [2]
Level			= Vars + 16		; [1]
Letter			= Vars + 17		; [1]
CharPos			= Vars + 18		; [1]
GBType			= Vars + 19		; [1]
Sprites			= Vars + $100	; [160]
Map				= Vars + $200	; [1024]

Checksum		= SaveRam		; [2]
Scores			= SaveRam + 2	; [15]	3 x (score[2], initials[3])

.ext DisplayACZLogo
.ext DisplayBungLogo
.ext DisplayBungLogoDMG

Main:
 ld hl,StackTop
 ld sp,hl

 cp $11
 ld hl,GBType
 jr nz,@notcolor 
 ld a,1
 ld (hl),a
 call DisplayBungLogo
 ld a,1
 call DisplayACZLogo
 jr Start
@notcolor:
 sub a
 ld (hl),a
 call DisplayBungLogoDMG

Start:
 call Setup
 call SetupScores

 ld a,1
 ld (Level),a
MainLoop:
 call ShowTitle
 call PlayGame
 jr MainLoop

;============================================================================
PlayGame:
 sub a
 ldh (SCX),a
 ldh (WX),a
 dec a
 ldh (SCY),a
 ld (MapRow),a
 ld a,136
 ldh (WY),a
 ld hl,_BG1
 ld bc,$400
 call _clearmem
 ld a,(GBType)
 or a
 jr z,@skipattr
 ld a,1
 ldh (VBK),a
 ld hl,_BG1
 ld bc,$400
 call _clearmem
 ldh (VBK),a
@skipattr:
 ld hl,Map
 ld bc,1024
 call _clearmem
 ld hl,PatternTile
 ld de,_WNPA
 ld bc,16
 call _ldir
 ld hl,BallY
 sub a
 ldi (hl),a
 ld a,76
 ldi (hl),a
 sub a
 ld (hl),a
 ld (Sprites+3),a
 ld hl,Score
 sub a
 ldi (hl),a
 ld (hl),a
 ld hl,ScoreText
 ld de,_WIN1+8
 call Puts
 ld hl,_WIN1+15
 ld bc,5
 call _clearmem
 ld a,LCD_ON+USE_WIN1+WIN_ON+OBJ_ON+BG_ON
 ldh (LCDC),a
PlayLoop:
 call GetKey
 ld hl,MapRow
 ld a,(hl)
 inc a
 ld (hl),a
 ldh (SCY),a
 and %00011111
 jp nz,MovePlayer
GenerateRow:
 ld a,(MapRow)
 rra
 rra
 rra
 add a,18
 and %00011111
 or a
 rla
 rla
 rla
 ld h,0
 ld l,a
 add hl,hl
 add hl,hl
 ld d,h
 ld e,l
 ld hl,_BG1
 add hl,de
 push hl 
 ld hl,Map
 add hl,de
 ld b,10
@loop:
 ld a,(Level)
 cp 1
 jr z,@normal
 call Random
 and %00000111
 jr nz,@placeleft
 jr @noplace
@normal:
 call Random
 and %00000011
 jr nz,@placeleft
@noplace: 
 sub a
 ldi (hl),a
 ld c,%00000001
 jr @doneleft
@placeleft:
 ld a,BLOCKSPR
 ldi (hl),a
 ld c,%00011111
@doneleft:
 call Random
 and c
 jr nz,@placeright
 sub a
 ldi (hl),a
 jr @doneright
@placeright:
 ld a,BLOCKSPR+1
 ldi (hl),a
@doneright:
 dec b
 jr nz,@loop
 ld de,-20
 add hl,de
 push hl
 ld bc,(20<<8)
@check:
 ldi a,(hl)
 or a
 jr z,@nothole
 inc c
@nothole:
 dec b
 jr nz,@check
 ld a,c
 cp 20
 jr nz,@holesexist
 ld de,-20
 add hl,de
 call Random
 and %00001111
 ld d,0
 ld e,a
 add hl,de
 sub a
 ld (hl),a
@holesexist:
 call WaitForVBlank
 pop hl
 pop de
 ld a,(GBType)
 or a
 jr z,@colorsdone
 push hl
 push de
 ld a,1
 ldh (VBK),a
 call Random
 and %00000111
 ld c,a
 ld b,20
@color:
 ldi a,(hl)
 or a
 jr z,@holecolor
 ld a,c
 ld (de),a
 jr @nextcolor
@holecolor:
 sub a
 ld (de),a
@nextcolor: 
 inc de
 dec b
 jr nz,@color
 sub a
 ldh (VBK),a
 pop de
 pop hl
@colorsdone:
 ld bc,20
 call _ldir
MovePlayer:
 call DoMovePlayer
 call DoMovePlayer
 ld a,(Level)
 or a
 jr nz,DropBall
 call DoMovePlayer
 call DoMovePlayer
DropBall:
; ld a,(Level)
; cp 1
; jr nz,@skip 
; ld a,(MapRow)
; ld b,a
; ld a,(BallY)
; add a,b
; and %11111110
; call DoDropBall
;@skip:
 ld a,(MapRow)
 ld b,a
 ld a,(BallY)
 add a,b
 call DoDropBall
DrawPlayer:
 ld hl,Sprites
 ld de,BallY
 ld a,(de)
 inc de
 add a,16
 ldi (hl),a
 ld a,(de)
 inc de
 add a,8
 ldi (hl),a
 ld a,(KeyP)
 rla
 call c,PauseGame
 halt
 jp PlayLoop

PauseGame:
 call GetKey
 ld a,b
 or a
 jr nz,PauseGame
 call WaitForVBlank 
 sub a
 ldh (LCDC),a
 ld hl,_WIN1
 ld bc,20
 call _clearmem
 ld hl,PauseText
 ld de,_WIN1+(32*9)+8
 call Puts
 sub a
 ldh (WY),a
 ld a,LCD_ON+USE_WIN1+WIN_ON+OBJ_ON+BG_ON
 ldh (LCDC),a
 call PauseStart
 sub a
 ldh (LCDC),a
 ld a,136
 ldh (WY),a
 ld hl,_WIN1+(32*9)+8
 ld bc,6
 call _clearmem
 ld hl,ScoreText
 ld de,_WIN1+8 
 call Puts
 ld a,LCD_ON+USE_WIN1+WIN_ON+OBJ_ON+BG_ON
 ldh (LCDC),a
 ret 

DoMovePlayer:
 ld hl,BallX
 ld a,(KeyP)
 rra
 jr nc,@notright
 ld a,(MapRow)
 and %00000111
 jr nz,@skipanimr
 push hl
 ld hl,BallAnim
 ld a,(hl)
 inc a
 and %00000011
 add a,8
 ld (hl),a
 pop hl
 ld (Sprites+2),a
@skipanimr:
 ld a,(hl)
 cp 152
 ret z
 ld a,(hl)
 and %00000111
 jr nz,@rightedge
 push hl
 ld a,(BallY)
 call GetBallTile
 inc hl
 ld a,(hl)
 or a
 pop hl
 ret nz
@rightedge:
 inc (hl)
 ret
@notright:
 rra
 ret nc
 ld a,(MapRow)
 and %00000111
 jr nz,@skipaniml
 push hl
 ld hl,BallAnim
 ld a,(hl)
 inc a
 and %00000011
 add a,4
 ld (hl),a
 pop hl
 ld (Sprites+2),a
@skipaniml:
 ld a,(hl)
 or a
 ret z
 ld a,(hl)
 and %00000111
 jr nz,@leftedge
 push hl
 ld a,(BallY)
 call GetBallTile
 dec hl
 ld a,(hl)
 or a
 pop hl
 ret nz
@leftedge:
 dec (hl)
 ret

DoDropBall:
 ld c,a
 and %00000111
 jr nz,@dodrop
 ld a,c
 add a,8
 call GetBallTile
 ldi a,(hl)
 or a
 jr nz,@catch
 ld a,(BallX)
 and %00000111
 jr z,@dodrop
 ld a,(hl)
 or a
 jr z,@dodrop
@catch:
 ld hl,BallY
 ld a,(hl)
 or a
 jr z,PlayerDied
 dec (hl)
 ret
@dodrop:
 ld a,(MapRow)
 and %00000111
 jr nz,@skipanim
 ld a,(KeyP)
 and %00000011
 jr nz,@skipanim
 ld hl,BallAnim
 ld a,(hl)
 inc a
 and %00000011
 ld (hl),a
 ld (Sprites+2),a
@skipanim:
 ld hl,BallY
 ld a,(hl)
 cp 128
 jr z,@skip
 inc (hl)
@skip:
 jr UpdateScore

PlayerDied:
 pop hl
 call Pause
 call WaitForVBlank
 sub a
 ldh (LCDC),a
 ld hl,_WNPA
 ld bc,16
 call _clearmem
 jp UpdateHigh

GetBallTile:
 and %11111000
 ld h,0
 ld l,a
 add hl,hl
 add hl,hl
 ld de,Map
 add hl,de
 ld a,(BallX)
 rra
 rra
 rra
 and %00011111
 ld d,0
 ld e,a
 add hl,de
 ret

UpdateScore:
 ld hl,Score
 ld e,(hl)
 inc hl
 ld d,(hl)
 inc de
 ld (hl),d
 dec hl
 ld (hl),e
 ld hl,_WIN1+15
 jp DispDE  

ScoreText:
 .db "Score:",0

PauseText:
 .db "PAUSED",0

PatternTile:
.db $11,$00,$AA,$00,$44,$00,$AA,$00
.db $11,$00,$AA,$00,$44,$00,$AA,$00

;============================================================================
Random:
 push hl
 push de
 ld hl,RandSeed
 ldi a,(hl)
 ld h,(hl)
 ld l,a
 ld de,103
 add hl,de
 ld a,l
 ld (RandSeed),a
 ld a,h
 ld (RandSeed+1),a
 cpl
 add a,l
 ld d,0
 ld e,a
 ld hl,RandTable
 add hl,de
 ld a,(hl)
 pop de
 pop hl
 ret 

RandTable:
 .db $69,$ce,$9b,$b4,$54,$a5,$9e,$cd,$57,$12,$4c,$65,$14,$4d,$92,$58
 .db $35,$8b,$a9,$05,$40,$dd,$ed,$bd,$0a,$1b,$bc,$b0,$c7,$15,$e7,$8c
 .db $4b,$88,$d7,$93,$50,$63,$d9,$fa,$af,$82,$7a,$00,$5a,$b9,$09,$bb
 .db $16,$75,$e3,$72,$f3,$c8,$ba,$d6,$e9,$37,$b6,$07,$e1,$c0,$79,$db
 .db $d3,$f2,$9a,$6a,$aa,$b5,$20,$56,$04,$5c,$e6,$59,$83,$2e,$95,$b7
 .db $c1,$5b,$28,$d4,$74,$9c,$5d,$66,$ec,$23,$67,$9d,$a1,$64,$0f,$2c
 .db $6c,$8e,$bf,$08,$99,$7f,$4a,$33,$b8,$6e,$fc,$6d,$a8,$36,$89,$5e
 .db $c4,$73,$b1,$d2,$32,$fe,$7b,$85,$55,$80,$53,$a4,$0e,$7e,$d0,$c5
 .db $f8,$18,$0b,$a6,$c2,$c9,$13,$dc,$94,$71,$eb,$86,$c3,$62,$84,$46
 .db $47,$90,$3f,$da,$a3,$6f,$06,$f5,$98,$ca,$8d,$2f,$e8,$9f,$60,$de
 .db $61,$fb,$ad,$3b,$f7,$3e,$4f,$0c,$f4,$e0,$d8,$ac,$d1,$31,$c6,$34
 .db $a2,$17,$8f,$ea,$2a,$5f,$96,$81,$45,$48,$f9,$30,$21,$68,$27,$f1
 .db $fd,$4e,$ee,$cc,$78,$77,$1d,$97,$1c,$24,$3d,$a0,$2d,$26,$03,$22
 .db $e5,$11,$ef,$6b,$b2,$e2,$02,$49,$3a,$d5,$87,$76,$39,$ab,$df,$1e
 .db $29,$cb,$1f,$52,$7c,$01,$3c,$38,$0d,$41,$f6,$1a,$2b,$ae,$f0,$10
 .db $91,$b3,$43,$7d,$42,$44,$70,$8a,$cf,$ff,$25,$a7,$be,$19,$51,$e4

;============================================================================
SetupScores:
 ld a,$a
 ld ($0000),a
 ld hl,Checksum
 ld de,CHECKSUM
 ldi a,(hl)
 cp e
 jr nz,@reset
 ld a,(hl)
 cp d
 jr z,@done  
@reset:
 ld de,CHECKSUM
 ld a,d
 ldd (hl),a
 ld a,e
 ldi (hl),a
 inc hl
 ld b,3
@loop:
 sub a
 ldi (hl),a
 ldi (hl),a
 ld a,'A'
 ldi (hl),a
 ld a,'C'
 ldi (hl),a
 ld a,'Z'
 ldi (hl),a 
 dec b
 jr nz,@loop
@done:
 sub a
 ld ($0000),a 
 ret

ShowTitle:
 sub a
 ldh (SCY),a
 ldh (SCX),a
 ld hl,_BG1
 ld bc,$400
 call _clearmem
 ld a,(GBType)
 or a
 jr z,@skipattr
 ld a,1
 ldh (VBK),a
 ld hl,_BG1
 ld bc,$400
 call _clearmem
 ld hl,_BG1+32
 ld bc,32*10
 ld a,1
 call _setram
 ld hl,_BG1+(32*14)+3
 ld bc,16
 ld a,2
 call _setram
 sub a
 ldh (VBK),a
 ld hl,TitlePalettes
 ld b,24
 call LoadBGPalettes
@skipattr:
 ld hl,_BG1+32
 ld de,TitleMap
 ld b,10
@copytitle:
 ld c,20
@copyrow:
 ld a,(de)
 inc de
 ldi (hl),a
 dec c
 jr nz,@copyrow
 ld a,b
 ld bc,12
 add hl,bc
 ld b,a
 dec b
 jr nz,@copytitle 
 ld hl,TitleText
 ld de,_BG1+(32*16)+4
 call Puts
 ld a,LCD_ON+BG_ON
 ldh (LCDC),a
SelectLevel:
 ld a,(Level)
 rla
 rla
 rla
 rla
 and %11110000
 ld d,0
 ld e,a
 ld hl,LevelText
 add hl,de
 ld de,_BG1+(32*14)+2
 call PutsW
 ld hl,HighText
 ld de,_BG1+(32*12)+1
 call PutsW
 ld a,$a
 ld ($0000),a
 ld a,(Level)
 or a
 jr z,@calcdone
 ld b,a
 sub a
@calc:
 add a,5
 dec b
 jr nz,@calc 
@calcdone:
 ld c,a
 ld hl,Scores
 add hl,bc
 ld e,(hl)
 inc hl
 ld d,(hl)
 inc hl
 push hl
 ld hl,_BG1+(32*12)+7
 call DispDE
 pop hl
 ld de,TempStr
 ld bc,3
 call _ldir
 ld (de),a
 ld hl,TempStr
 ld de,_BG1+(32*12)+16
 call PutsW
@loop:
 halt
 call GetKey
 ld hl,Level
 ld a,b
 rra
 jr nc,@notright
 ld a,(hl)
 cp 2
 jr z,@loop
 inc (hl)
@rightup:
 halt
 call GetKey
 ld a,b
 rra
 jr c,@rightup
 jp SelectLevel 
@notright:
 rra
 jr nc,@notleft
 ld a,(hl)
 or a
 jr z,@loop
 dec (hl)
@leftup:
 halt
 call GetKey
 ld a,b
 rra
 rra
 jr c,@leftup 
 jp SelectLevel 
@notleft:
 ld a,(KeyP)
 rla
 jr nc,@loop
@release:
 call GetKey
 ld a,b
 or a
 jr nz,@release
 ld ($0000),a

TitleClear:
 call WaitForVBlank
 sub a
 ldh (LCDC),a
 ld hl,_BG1
 ld bc,$400
 call _clearmem
 ld a,(GBType)
 or a
 jr z,@skipattr
 ld a,1
 ldh (VBK),a
 ld hl,_BG1
 ld bc,$400
 call _clearmem
 ldh (VBK),a
 ld hl,ColorPalettes
 ld b,24
 call LoadBGPalettes
@skipattr:
 ret

TitleText:
 .db "Push Start!",0

HighText:
 .db "High:       by    ",0

; each line is 16 bytes to make it easy to calculate offset
LevelText:
 .db "     Easy    } ",0
 .db " {  Normal   } ",0
 .db " {   Hard      ",0

TitlePalettes:
 .dw 0+(32*0)+(1024*0)		; R:0,G:0,B:0
 .dw 21+(32*31)+(1024*20)	; R:20,G:31,B:20
 .dw 10+(32*31)+(1024*10)	; R:10,G:31,B:10
 .dw 0+(32*31)+(1024*0)		; R:0,G:31,B:0

 .dw 0+(32*0)+(1024*0)		; R:0,G:0,B:0
 .dw 21+(32*20)+(1024*31)	; R:20,G:20,B:31
 .dw 10+(32*10)+(1024*31)	; R:10,G:10,B:31
 .dw 0+(32*0)+(1024*31)		; R:0,G:0,B:31

 .dw 0+(32*0)+(1024*0)		; R:0,G:0,B:0
 .dw 31+(32*20)+(1024*20)	; R:31,G:20,B:20
 .dw 31+(32*10)+(1024*10)	; R:31,G:10,B:10
 .dw 31+(32*0)+(1024*0)		; R:31,G:0,B:0

TitleMap:
.db $80,$81,$82,$83,$84,$85,$86,$8D,$8E,$8F
.db $90,$91,$92,$80,$80,$80,$80,$80,$80,$80
.db $80,$87,$88,$89,$8A,$8B,$8C,$93,$94,$95
.db $96,$97,$98,$80,$80,$CF,$D0,$80,$80,$80
.db $80,$80,$80,$80,$80,$80,$80,$99,$9A,$9B
.db $9C,$9D,$9E,$80,$80,$D1,$D2,$80,$80,$80
.db $80,$80,$80,$80,$80,$80,$80,$9F,$A0,$A1
.db $A2,$A3,$A4,$80,$80,$80,$80,$80,$80,$80
.db $80,$80,$80,$80,$80,$80,$80,$A5,$A6,$A7
.db $A8,$A9,$AA,$80,$80,$80,$80,$80,$80,$80
.db $80,$80,$80,$80,$80,$80,$80,$AB,$AC,$AD
.db $AE,$AF,$B0,$80,$80,$80,$80,$80,$80,$80
.db $80,$80,$CF,$D0,$80,$80,$80,$B1,$B2,$B3
.db $B4,$B5,$B6,$C3,$C4,$C5,$C6,$C7,$C8,$80
.db $80,$80,$D1,$D2,$80,$80,$80,$B7,$B8,$B9
.db $BA,$BB,$BC,$C9,$CA,$CB,$CC,$CD,$CE,$80
.db $80,$80,$80,$80,$80,$80,$80,$BD,$BE,$BF
.db $C0,$C1,$C2,$80,$80,$80,$80,$80,$80,$80
.db $80,$80,$80,$80,$80,$80,$80,$80,$80,$80
.db $80,$80,$80,$80,$80,$80,$80,$80,$80,$80

;============================================================================
UpdateHigh:
 ld a,$a
 ld ($0000),a
 ld a,(Level)
 or a
 jr z,@calcdone
 ld b,a
 sub a
@calc:
 add a,5
 dec b
 jr nz,@calc 
@calcdone:
 ld c,a
 ld hl,Scores
 add hl,bc
 push hl
 ldi a,(hl)
 ld d,(hl)
 ld e,a
 ld hl,Score
 ldi a,(hl)
 ld h,(hl)
 ld l,a
 sub a
 ld ($0000),a
 ld a,h
 cp d
 jr c,@toolow
 jr nz,@gothigh
 ld a,e
 cp l
 jr c,@gothigh
@toolow:
 pop hl 
 ret

@gothigh:
 sub a
 ldh (SCY),a
 ld a,-4
 ldh (SCX),a
 ld hl,_BG1
 ld bc,$400
 call _clearmem
 ld a,(GBType)
 or a
 jr z,@skipattr
 ld a,1
 ldh (VBK),a
 ld hl,_BG1
 ld bc,$400
 call _clearmem
 ld hl,_BG1+32
 ld bc,32*10
 ld a,1
 call _setram
 ld hl,_BG1+(32*9)+13
 ld bc,5
 ld a,2
 call _setram 
 sub a
 ldh (VBK),a
 ld hl,TitlePalettes
 ld b,24
 call LoadBGPalettes
 ld hl,RedPalette
 ld b,8
 call LoadOPalettes
@skipattr:

 ld hl,UpdateHighText
 ld de,_BG1+(32*1)+2
 call Puts
 ld de,_BG1+(32*3)+1
 call Puts
 ld de,_BG1+(32*4)+1
 call Puts
 ld de,_BG1+(32*9)+3
 call Puts
 ld de,_BG1+(32*12)
 call Puts
 ld de,_BG1+(32*13)
 call Puts
 ld de,_BG1+(32*14)
 call Puts
 ld de,_BG1+(32*15)
 call Puts
 ld de,_BG1+(32*16)
 call Puts

 sub a
 ld (Letter),a
 ld (CharPos),a
 ld a,CURSORSPR
 ld (Sprites+2),a

 ld a,LCD_ON+OBJ_ON+BG_ON
 ldh (LCDC),a

@entryloop:
 ld a,(CharPos)
 add a,a
 ld d,0
 ld e,a
 ld hl,CharSpriteTable
 add hl,de
 ldi a,(hl)
 ld (Sprites),a
 ld a,(hl)
 ld (Sprites+1),a
@loop:
 halt
 call GetKey
 and b
 jr z,@loop
 ld hl,CharPos
 rra
 jr nc,@notright
 ld a,(hl)
 cp 26
 jr z,@loop
 inc (hl)
 jp @release
@notright:
 rra
 jr nc,@notleft
 ld a,(hl)
 or a
 jr z,@loop
 dec (hl)
 jp @release
@notleft:
 rra
 jr nc,@notup
 ld a,(hl)
 cp 9
 jr c,@wrapup
 sub 9
 ld (hl),a
 jp @release
@wrapup:
 add a,18
 ld (hl),a
 jr @release
@notup:
 rra
 jr nc,@notdown
 ld a,(hl)
 cp 18
 jr nc,@wrapdown
 add a,9
 ld (hl),a
 jr @release
@wrapdown:
 sub 18
 ld (hl),a
 jr @release
@notdown:
 rra
 jr nc,@notent
 ld a,(CharPos)
 cp 26
 jr z,@done
 add a,'A'
 ld b,a
 ld a,(Letter)
 cp 3
 jr z,@release
 ld c,a
 ld d,0
 ld e,a
 ld hl,TempStr
 add hl,de
 ld a,b
 ldi (hl),a
 ld a,c
 cp 2
 jr z,@skipcursor
 ld a,'~'
 ldi (hl),a
@skipcursor:
 sub a
 ld (hl),a
 inc c
 ld a,c
 ld (Letter),a
 ld hl,TempStr
 ld de,_BG1+(32*9)+13
 call PutsW  
 jr @release
@notent:
 rra 
 jr nc,@notback
 ld a,(Letter)
 or a
 jr z,@release
 dec a
 ld (Letter),a
 ld d,0
 ld e,a
 ld hl,TempStr
 add hl,de
 ld a,'~'
 ldi (hl),a
 sub a
 ld (hl),a
 ld hl,BlankInitialsText
 ld de,_BG1+(32*9)+13
 call PutsW 
 ld hl,TempStr
 ld de,_BG1+(32*9)+13
 call PutsW  
 jr @release
@notback:
 rra
 rra
 jp nc,@loop
 jr @done 
@release:
 halt
 call GetKey
 and b
 jr nz,@release
 jp @entryloop
@done:
 ld a,(Letter)
 ld d,0
 ld e,a
 ld hl,TempStr
 add hl,de
 sub a
 ldi (hl),a
 ldi (hl),a
 ld (hl),a
 ld a,$a
 ld ($0000),a 
 pop de
 ld hl,Score
 ldi a,(hl)
 ld h,(hl)
 ld (de),a
 inc de
 ld a,h
 ld (de),a
 inc de
 ld hl,TempStr
 ld bc,3
 call _ldir 
 sub a
 ld ($0000),a
@wait:
 call GetKey
 ld a,b
 or a
 jr nz,@wait
 call WaitForVBlank
 sub a
 ldh (LCDC),a
 ld a,(GBType)
 or a
 ret z
 ld hl,ColorPalettes
 ld b,24
 call LoadBGPalettes
 ld hl,BlackPalette
 ld b,8
 call LoadOPalettes
 ret

BlankInitialsText:
 .db "   ",0

UpdateHighText:
 .db "Congratulations!",0
 .db "You just beat the",0
 .db "high score!",0
 .db "Initials:",0
 .db $82,$86,$86,$86,$86,$86,$86,$86,$86,$86,$86,$86,$86,$86,$86,$86,$86,$86,$84,0
 .db $88,"A B C D E F G H I",$89,0
 .db $88,"J K L M N O P Q R",$89,0
 .db $88,"S T V U W X Y Z |",$89,0
 .db $83,$87,$87,$87,$87,$87,$87,$87,$87,$87,$87,$87,$87,$87,$87,$87,$87,$87,$85,0

CharSpriteTable:
 .db 124,30, 124,46, 124,62, 124,78, 124,94, 124,110, 124,126, 124,142, 124,158
 .db 132,30, 132,46, 132,62, 132,78, 132,94, 132,110, 132,126, 132,142, 132,158
 .db 140,30, 140,46, 140,62, 140,78, 140,94, 140,110, 140,126, 140,142, 140,158

;============================================================================
Setup:
 ld a,$d9				 ; reti
 ldh ($83),a
 ei
 ld a,%00000001
 ldh (IE),a

 call WaitForVBlank		; wait for vblank to turn off lcd
 sub a
 ldh (LCDC),a

; sub a					; swap in bank 0, copy font sprites
 ldh (VBK),a
 ld hl,FontChars
 ld de,_WNPA
 ld bc,16*$60
 call _ldir

 ld hl,BlockSprites		; copy block/box sprites to tile memory
 ld de,_WNPA+(16*$60)
 ld bc,(16*2)+(16*8)
 call _ldir  

 ld hl,BallSprites		; copy ball and cursor sprites to object tile memory
 ld de,_OBPA
 ld bc,(16*12)+(16*1)
 call _ldir

 ld hl,TitleTiles		; copy title tiles to background memory
 ld de,_BGPA
 ld bc,16*83
 call _ldir

 ld hl,Sprites			; clear sprite memory
 ld bc,4*40
 call _clearmem

 ld hl,VBlankIntr		; copy DMA handler to high ram
 ld de,$ff83
 ld bc,VBlankIntrEnd-VBlankIntr
 call _ldir

 ld a,(GBType)
 or a
 jr z,@notcolor
 ld hl,ColorPalettes	; set bg/window palettes
 ld b,8*8
 call LoadBGPalettes

 ld hl,BlackPalette		; set object palette 0 to gray
 ld b,8
 call LoadOPalettes
@notcolor:

 ld a,%11100100			; set DMG palettes to grayscale
 ldh (BGP),a			; set bg/window
 ldh (OBP0),a			; set object palette 0

 ld a,%00000001			; enable vblank, disable all other interrupts
 ldh (IE),a

 ret

VBlankIntr:
 push af
 ld a,Sprites>>8
 ldh (DMA),a
 ld a,$28
@wait:
 dec a
 jr nz,@wait
 pop af
 reti
VBlankIntrEnd

ColorPalettes:

BlackPalette:
 .dw 31+(32*31)+(1024*31)	; R:31,G:31,B:31
 .dw 20+(32*20)+(1024*20)	; R:20,G:20,B:20
 .dw 10+(32*10)+(1024*10)	; R:10,G:10,B:10
 .dw 0+(32*0)+(1024*0)		; R:0,G:0,B:0

RedPalette:
 .dw 31+(32*31)+(1024*31)	; R:31,G:31,B:31
 .dw 31+(32*20)+(1024*20)	; R:31,G:20,B:20
 .dw 31+(32*10)+(1024*10)	; R:31,G:10,B:10
 .dw 31+(32*0)+(1024*0)		; R:31,G:0,B:0

GreenPalette:
 .dw 31+(32*31)+(1024*31)	; R:31,G:31,B:31
 .dw 21+(32*31)+(1024*20)	; R:20,G:31,B:20
 .dw 10+(32*31)+(1024*10)	; R:10,G:31,B:10
 .dw 0+(32*31)+(1024*0)		; R:0,G:31,B:0

BluePalette:
 .dw 31+(32*31)+(1024*31)	; R:31,G:31,B:31
 .dw 21+(32*20)+(1024*31)	; R:20,G:20,B:31
 .dw 10+(32*10)+(1024*31)	; R:10,G:10,B:31
 .dw 0+(32*0)+(1024*31)		; R:0,G:0,B:31

LightBluePalette:
 .dw 31+(32*31)+(1024*31)	; R:31,G:31,B:31
 .dw 20+(32*31)+(1024*31)	; R:20,G:31,B:31
 .dw 10+(32*31)+(1024*31)	; R:10,G:31,B:31
 .dw 0+(32*31)+(1024*31)	; R:0,G:31,B:31

OrangePalette:
 .dw 31+(32*31)+(1024*31)	; R:31,G:31,B:31
 .dw 31+(32*27)+(1024*10)	; R:31,G:27,B:10
 .dw 31+(32*22)+(1024*5)	; R:31,G:22,B:5
 .dw 31+(32*17)+(1024*0)	; R:31,G:17,B:0

PurplePalette:
 .dw 31+(32*31)+(1024*31)	; R:31,G:31,B:31
 .dw 31+(32*20)+(1024*31)	; R:31,G:20,B:31
 .dw 28+(32*10)+(1024*28)	; R:28,G:10,B:28
 .dw 23+(32*0)+(1024*23)	; R:23,G:0,B:23

BrownPalette:
 .dw 31+(32*31)+(1024*31)	; R:31,G:31,B:31
 .dw 27+(32*26)+(1024*20)	; R:27,G:26,B:20
 .dw 23+(32*20)+(1024*10)	; R:23,G:20,B:10
 .dw 19+(32*14)+(1024*0)	; R:19,G:14,B:0

BlockSprites:
.db $FF,$FF,$FE,$FF,$FD,$FE,$FA,$FD
.db $F5,$FA,$EA,$F5,$D5,$EA,$FF,$FF
.db $FF,$FF,$AB,$57,$57,$AF,$AF,$5F
.db $5F,$BF,$BF,$7F,$7F,$FF,$FF,$FF

BoxTiles:
.db $00,$00,$00,$00,$3F,$3F,$3F,$7F
.db $B0,$70,$B0,$70,$B0,$70,$B0,$70
.db $B0,$70,$B0,$70,$B0,$70,$B0,$70
.db $BF,$7F,$BF,$7F,$80,$7F,$FF,$00
.db $00,$00,$00,$00,$FC,$FC,$FC,$FC
.db $0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C
.db $0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C
.db $FC,$FC,$FC,$FC,$00,$F8,$F0,$00
.db $00,$00,$00,$00,$FF,$FF,$FF,$FF
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $FF,$FF,$FF,$FF,$00,$FF,$FF,$00
.db $B0,$70,$B0,$70,$B0,$70,$B0,$70
.db $B0,$70,$B0,$70,$B0,$70,$B0,$70
.db $0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C
.db $0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C

BallSprites:
.db $3C,$3C,$4E,$42,$8F,$81,$9D,$83
.db $FD,$83,$F9,$87,$62,$5E,$3C,$3C
.db $3C,$3C,$7E,$7E,$E1,$FF,$D9,$C7
.db $FD,$83,$F9,$87,$62,$5E,$3C,$3C
.db $3C,$3C,$4E,$42,$B7,$99,$FF,$FF
.db $FF,$FF,$F9,$9F,$62,$5E,$3C,$3C
.db $3C,$3C,$4E,$42,$8F,$81,$9D,$83
.db $E1,$9F,$CF,$FF,$7E,$7E,$3C,$3C
.db $3C,$3C,$72,$42,$F1,$81,$B9,$C1
.db $BF,$C1,$9F,$E1,$46,$7A,$3C,$3C
.db $3C,$3C,$7E,$4E,$F7,$87,$BB,$C7
.db $BB,$C7,$93,$EF,$42,$7E,$3C,$3C
.db $3C,$3C,$7A,$5A,$FD,$99,$BD,$FD
.db $BB,$FD,$9F,$F9,$5E,$7A,$3C,$3C
.db $3C,$3C,$72,$62,$D1,$E1,$C9,$F1
.db $EF,$F1,$EF,$F1,$66,$7A,$3C,$3C
.db $3C,$3C,$62,$5E,$F9,$87,$FD,$83
.db $9D,$83,$8F,$81,$4E,$42,$3C,$3C
.db $3C,$3C,$42,$7E,$C9,$F7,$DD,$E3
.db $DD,$E3,$EF,$E1,$7E,$72,$3C,$3C
.db $3C,$3C,$7A,$5E,$F9,$9F,$DD,$BF
.db $BD,$BF,$BF,$99,$5E,$5A,$3C,$3C
.db $3C,$3C,$66,$5E,$F7,$8F,$F7,$8F
.db $93,$8F,$8B,$87,$4E,$46,$3C,$3C

CursorSprite:
.db $F0,$F0,$E0,$E0,$F0,$F0,$B8,$B8
.db $10,$10,$00,$00,$00,$00,$00,$00

TitleTiles:
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$7F,$7F,$60,$7F,$40,$60
.db $40,$60,$7F,$7F,$60,$7F,$40,$60
.db $00,$00,$80,$C0,$40,$80,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$F8,$FC,$0C,$FE,$06,$06
.db $00,$00,$00,$00,$0F,$07,$01,$07
.db $00,$01,$00,$01,$00,$01,$00,$01
.db $00,$00,$00,$00,$80,$80,$80,$80
.db $80,$80,$80,$80,$80,$80,$80,$80
.db $00,$00,$00,$00,$7C,$3C,$0C,$3C
.db $04,$0C,$04,$0C,$04,$0C,$04,$0C
.db $40,$60,$40,$60,$40,$60,$40,$60
.db $40,$60,$40,$60,$40,$60,$00,$00
.db $00,$00,$00,$01,$01,$03,$02,$03
.db $02,$03,$01,$03,$00,$01,$00,$00
.db $02,$06,$FE,$FE,$8E,$FE,$02,$06
.db $0E,$06,$9E,$FE,$F6,$FA,$00,$00
.db $00,$01,$00,$01,$00,$01,$00,$01
.db $00,$01,$01,$07,$07,$07,$00,$00
.db $80,$80,$80,$80,$80,$80,$80,$80
.db $80,$80,$90,$E0,$F0,$E0,$00,$00
.db $04,$0C,$04,$0C,$04,$0C,$04,$0C
.db $04,$0C,$0C,$3F,$3F,$3F,$00,$00
.db $00,$00,$00,$00,$06,$01,$18,$07
.db $30,$0F,$20,$1F,$60,$1F,$40,$3F
.db $0F,$00,$E0,$1F,$00,$FF,$00,$FF
.db $00,$FF,$00,$FF,$00,$FF,$00,$FF
.db $80,$7F,$00,$FF,$00,$FF,$00,$FF
.db $00,$FF,$00,$FF,$00,$FF,$00,$FF
.db $3E,$C0,$00,$FF,$00,$FF,$00,$FF
.db $00,$FF,$00,$FF,$00,$FF,$00,$FF
.db $00,$00,$60,$80,$0C,$F0,$03,$FC
.db $01,$FE,$00,$FF,$00,$FF,$00,$FF
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $80,$00,$80,$00,$C0,$00,$C0,$00
.db $80,$7F,$BC,$7F,$BF,$7F,$BF,$7F
.db $BF,$7F,$BF,$7F,$BF,$7F,$BF,$7F
.db $00,$FF,$00,$FF,$80,$FF,$FC,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $00,$FF,$00,$FF,$00,$FF,$00,$FF
.db $FD,$F8,$F0,$F8,$F8,$F0,$F0,$F0
.db $00,$FF,$00,$FF,$00,$FF,$03,$FF
.db $FF,$FF,$7F,$FF,$FF,$7F,$7F,$7F
.db $00,$FF,$06,$FF,$1F,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $C0,$00,$40,$80,$40,$80,$40,$80
.db $40,$80,$40,$80,$40,$80,$40,$80
.db $BF,$7F,$BF,$7F,$BF,$7F,$BF,$7F
.db $BF,$7F,$BF,$7F,$BF,$7F,$BF,$7F
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $E0,$F0,$F0,$E0,$E0,$E0,$C0,$E0
.db $C0,$E0,$E0,$C0,$E0,$C0,$C0,$C0
.db $3F,$7F,$7F,$3F,$3F,$3F,$1F,$3F
.db $1F,$3F,$3F,$1F,$3F,$1F,$1F,$1F
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $40,$80,$40,$80,$60,$80,$60,$80
.db $60,$80,$60,$80,$60,$80,$60,$80
.db $DF,$3F,$DF,$3F,$DF,$3F,$DF,$3F
.db $DF,$3F,$DF,$3F,$DF,$3F,$DF,$3F
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $C0,$C0,$C0,$C0,$80,$C0,$80,$C0
.db $80,$C0,$C0,$80,$C0,$80,$C0,$80
.db $1F,$1F,$1F,$1F,$0F,$1F,$0F,$1F
.db $0F,$1F,$1F,$0F,$1F,$0F,$1F,$0F
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $60,$80,$60,$80,$60,$80,$60,$80
.db $60,$80,$60,$80,$60,$80,$60,$80
.db $DF,$3F,$DF,$3F,$DF,$3F,$DF,$3F
.db $DF,$3F,$DF,$3F,$DF,$3F,$DF,$3F
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FE,$FF,$FF,$FF,$FF,$FF
.db $D8,$80,$C0,$80,$80,$80,$00,$80
.db $00,$00,$FF,$00,$FF,$FF,$FF,$FF
.db $DF,$0F,$1F,$0F,$0F,$0F,$07,$0F
.db $07,$07,$EF,$1F,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $60,$80,$60,$80,$60,$80,$60,$80
.db $60,$80,$60,$80,$60,$80,$E0,$80
.db $DF,$3F,$DF,$3F,$DF,$3F,$DF,$3F
.db $DF,$3F,$DF,$3F,$DF,$3F,$5F,$3F
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$BF,$D7,$8F,$C0,$80
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$EF,$5F,$8F,$1F,$0F
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $E0,$80,$E0,$80,$E0,$80,$E0,$80
.db $E0,$80,$A0,$C0,$A0,$C0,$A0,$C0
.db $5F,$3F,$5F,$3F,$5F,$3F,$5F,$3F
.db $5F,$3F,$5F,$3F,$5F,$3F,$5F,$3F
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $C0,$80,$80,$C0,$80,$C0,$80,$C0
.db $C0,$C0,$C0,$C0,$C0,$C0,$E0,$C0
.db $1F,$0F,$0F,$1F,$0F,$1F,$0F,$1F
.db $1F,$1F,$1F,$1F,$1F,$1F,$3F,$1F
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $A0,$C0,$A0,$C0,$A0,$C0,$A0,$C0
.db $A0,$C0,$A0,$C0,$A0,$C0,$A0,$C0
.db $5F,$3F,$5F,$3F,$5F,$3F,$1E,$3F
.db $18,$3F,$00,$3F,$00,$3F,$30,$0F
.db $FF,$FF,$FC,$FF,$C0,$FF,$01,$FF
.db $0F,$FF,$3F,$FF,$3F,$FF,$3F,$FF
.db $E0,$C0,$20,$C0,$20,$C0,$E0,$E0
.db $F0,$E0,$E0,$F0,$F0,$F0,$F8,$F0
.db $3F,$1F,$21,$1F,$20,$1F,$3E,$3F
.db $7F,$3F,$3F,$7F,$7F,$7F,$FF,$7F
.db $FF,$FF,$FF,$FF,$0F,$FF,$00,$FF
.db $C0,$FF,$F0,$FF,$F0,$FF,$F0,$FF
.db $A0,$C0,$A0,$C0,$A0,$C0,$20,$C0
.db $20,$C0,$20,$C0,$20,$C0,$20,$C0
.db $18,$07,$04,$03,$03,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $0F,$FF,$01,$FF,$00,$FF,$70,$0F
.db $07,$00,$00,$00,$00,$00,$00,$00
.db $F0,$F8,$F8,$FD,$00,$FF,$00,$FF
.db $C0,$3F,$00,$00,$00,$00,$00,$00
.db $7F,$FF,$FE,$FF,$00,$FF,$00,$FF
.db $0F,$F0,$00,$00,$00,$00,$00,$00
.db $C0,$FF,$00,$FF,$03,$FC,$38,$C0
.db $80,$00,$00,$00,$00,$00,$00,$00
.db $60,$80,$80,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $F8,$F8,$FC,$4C,$6E,$46,$66,$42
.db $66,$42,$63,$41,$63,$41,$63,$41
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $03,$03,$0F,$04,$0C,$04,$0C,$04
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $E0,$E0,$F8,$18,$18,$08,$18,$08
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $60,$20,$60,$20,$61,$20,$6D,$20
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $83,$00,$83,$81,$83,$81,$83,$81
.db $00,$00,$00,$00,$00,$00,$00,$00
.db $7C,$00,$FE,$FE,$C6,$C2,$06,$02
.db $63,$41,$66,$42,$66,$42,$6E,$46
.db $FC,$FC,$F8,$00,$00,$00,$00,$00
.db $0C,$04,$0C,$04,$0C,$04,$0C,$04
.db $0F,$07,$03,$00,$00,$00,$00,$00
.db $18,$08,$18,$08,$18,$08,$18,$08
.db $F8,$F8,$E0,$00,$00,$00,$00,$00
.db $6D,$24,$6D,$24,$6F,$24,$7F,$3F
.db $3F,$11,$33,$11,$00,$00,$00,$00
.db $83,$81,$83,$81,$83,$81,$83,$81
.db $83,$01,$03,$01,$00,$00,$00,$00
.db $06,$02,$06,$02,$06,$02,$06,$02
.db $06,$02,$06,$02,$00,$00,$00,$00
.db $03,$00,$1C,$00,$20,$00,$41,$00
.db $47,$00,$FF,$00,$FF,$00,$FF,$FF
.db $80,$00,$70,$00,$78,$00,$FC,$00
.db $FC,$00,$FE,$00,$F4,$0E,$FE,$FE
.db $FF,$FF,$FF,$FF,$7F,$FF,$FF,$7F
.db $7F,$7F,$7F,$3F,$3F,$1F,$0B,$07
.db $FE,$FE,$FE,$FE,$FC,$FE,$FE,$FC
.db $FC,$FC,$FC,$F8,$F8,$F0,$A0,$C0

;============================================================================
; hl = source, de = dest, bc = bytes
_ldir:
 ldi a,(hl)
 ld (de),a
 inc de
 dec bc
 ld a,b
 or c
 jr nz,_ldir
 ret

; hl = dest, bc = bytes
_clearmem:
 sub a
 ldi (hl),a
 dec bc
 ld a,b
 or c
 jr nz,_clearmem
 ret

; hl = dest, bc = bytes, a = char
_setram:
 push de
 ld d,a
@loop:
 ldi (hl),a
 dec bc
 ld a,b
 or c
 ld a,d
 jr nz,@loop
 pop de
 ret

WaitForVBlank:
 ldh a,(LY)
 cp 145
 jr nz,WaitForVBlank
 ret

WaitForHBlank:
 ldh a,(STAT)
 and %00000011
 jr nz,WaitForHBlank
 ret

PauseStart:
 halt
 call GetKey
 ld a,b
 or a
 jr nz,PauseStart
@wait:
 halt
 call GetKey
 ld a,b
 rla
 jr nc,@wait
@release:
 halt
 call GetKey
 ld a,b
 or a
 jr nz,@release
 ret

Pause:
 halt
 call GetKey
 ld a,b
 or a
 jr nz,Pause
@wait:
 halt
 call GetKey
 ld a,b
 and %11110000
 jr z,@wait
@release:
 halt
 call GetKey
 ld a,b
 or a
 jr nz,@release
 ret

;================================================
; GET KEYPRESSES
; keypresses stored in (KeyP)
; change in keypresses stored in (KeyC)
; bit 7 - start
; bit 6 - select
; bit 5 - B
; bit 4 - A
; bit 3 - down
; bit 2 - up
; bit 1 - left
; bit 0 - right
;================================================
GetKey:						;gets keypresses
 ld a,P15
 ldh (P1),a					;set P15
 nop \ nop \ nop
 nop \ nop \ nop
 ldh a,(P1)					;read control keys
 and %1111
 swap a
 ld b,a						;save those four bits for later
 ld a,P14
 ldh (P1),a					;set P15
 nop \ nop \ nop
 nop \ nop \ nop
 nop \ nop \ nop
 nop \ nop \ nop
 ldh a,(P1)					;read arrow keys
 and %1111					;keep the keypresses
 or b						;add control key data
 cpl						;make bits set if keypress
 ld b,a						;put it back in b
 ld a,P0
 ldh (P1),a					;deselect ports
 ld hl,KeyP
 ldi a,(hl)					;get old keypress
 xor b						;find changes
 ldd (hl),a					;store changes
 ld (hl),b					;store current keypress
 push af
 ld a,b						;handle 4-button reset
 and %11110000
 cp %11110000
 jr z,@reset
 pop af
 ret
@reset:
 pop af
 pop hl						;wait for release of all keys to reset
@release:
 call GetKey
 ld a,b
 or a
 jr nz,@release
 ld a,$11					;set color flag, like boot
 jp $150

; de = number, hl = memory to unpack to
UnpackDE:
 ld (hl),0
 dec hl
@loop:
 ld c,10
 call @divide
 add a,'0'
 ldd (hl),a					;store remainder into string
 ld a,d
 or e						;test DE
 jr nz,@loop
 inc hl
 ret
@divide:
 xor a
 ld b,16		;16 shifts
@dloop:
 sla e
 rl d 			;shift DE left
 rla 			;into A
 cp c 			;if A<=C
 jr c,@skip
 sub c			;then A= A-C
 inc e			;set bit 0
@skip:
 dec b
 jr nz,@dloop
 ret

; hl = palette data, b = bytes to set
LoadBGPalettes:
 ld a,%10000000				;select auto increment and palette 0, color 0
 ldh (BCPS),a
@loop:
 ldi a,(hl)
 ldh (BCPD),a				;copy palette data from HL
 dec b						;B times
 jr nz,@loop
 ret

; hl = palette data, b = bytes to set
LoadOPalettes:
 ld a,%10000000				;select auto increment and palette 0, color 0
 ldh (OCPS),a
@loop:
 ldi a,(hl)
 ldh (OCPD),a				;copy palette data from HL
 dec b						;B times
 jr nz,@loop
 ret

; de = number, hl = tile memory position
DispDE:
 push hl
 ld hl,TempStr+5
 call UnpackDE
 pop de

; hl = text, de = tile memory
; adjust sprite numbers for text
; waits for hblank before each character
PutsW:
 call WaitForHBlank
 ldi a,(hl)
 or a
 ret z
 sub $20
 ld (de),a
 inc de
 jr PutsW

; hl = text, de = tile memory
; adjust sprite numbers for text
Puts:
 ldi a,(hl)
 or a
 ret z
 sub $20
 ld (de),a
 inc de
 jr Puts

;============================================================================
#include "charfont.asm"

